package kalender;

import java.util.*;
import java.text.*;
import java.beans.*;
import java.io.*;

public class Kalender
{
	private boolean ndret;
	private List liste;
	/**
	 * @param kal Datoen vi nsker at kende indekset i listen p
	 * @returns indekset der skal bruges i kald til st() og hent().
	 */
	public int beregnIndex(Calendar kal) {
		int r = kal.get(Calendar.YEAR);
		int dag = kal.get(Calendar.DAY_OF_YEAR);
		// det vigtigste er at to dage aldrig fr samme indeks
		return (r-2003)*366+dag;
	}

	/**
	 * St teksten for en bestemt dag.
	 * @see #beregnIndeks(Calendar)
	 * @param indeks Indekset i listen. Skal frst findes med beregnIndeks()
	 * @param tekst Teksten for dagen
	 */
	public void stDagstekst(int indeks, String tekst) {
		// Fyld op med tomme strenge hvis der gs ud over listen
		while (indeks>=liste.size()) liste.add("");
		liste.set(indeks,tekst);
		ndret = true;
	}

	/**
	 * Hent teksten for en bestemt dag.
	 * @see #beregnIndeks(GregorianCalendar)
	 * @param index Indekset i listen. Skal frst findes med beregnIndeks()
	 * @return tekst Teksten for dagen
	 */
	public String hentDagstekst(int indeks) {
		if (indeks<0 || liste.size()<=indeks) return "";
		else return (String) liste.get(indeks);
	}

	public static final Kalender instans = new Kalender(); // singleton med

	private Kalender() {                                   // privat konstruktr
		try { // indls kalenderen fra XML-fil p disken (hvis den findes)
			XMLDecoder kal = new XMLDecoder(new FileInputStream("kalender.xml"));
			// alternativ: hent serialiseret objekt i stedet for XML-data
			//ObjectInputStream kal = new ObjectInputStream(
			//                        new FileInputStream("kalender.ser"));
			liste = (ArrayList) kal.readObject();
			kal.close();
			System.out.println("Kalender indlst: "+liste);
		} catch (Exception e) {
			System.out.println("Kalender ikke indlst, opretter ny: "+e);
			liste = new ArrayList();
		}
		GemRegelmssigt g = new GemRegelmssigt(); // gemmer kalenderen p disken
	}

	/** Srger for at gemme kalenderen regelmssigt (i en separat trd) */
	class GemRegelmssigt extends Thread
	{
		public GemRegelmssigt() {
			setDaemon(true);   // systemet m godt stoppe selvom trden stadig krer
			setPriority(MIN_PRIORITY);
			start();
		}

		public void run() {
			while (true) try {
				Thread.sleep(1*60*1000); // hvert minut,
				if (ndret) {            // hvis kalenderen er ndret
					ndret = false;        // gem kalenderen p disken
					// gem som XML
					XMLEncoder kal = new XMLEncoder(new FileOutputStream("kalender.xml"));
					// alternativ: gem som serialiseret objekt i stedet for XML
					//ObjectOutputStream kal = new ObjectOutputStream(
					//                         new FileOutputStream("kalender.ser"));
					kal.writeObject(liste);
					kal.close();
					System.out.println("Kalender gemt: "+liste);
				}
			} catch (Exception e) { e.printStackTrace(); }
		}
	}
}